import os.path
import sys
import string
import re

##############################################################################
# HEBREW AND GREEK LANGUAGE UTILITIES
##############################################################################
#
# contact@SpiritAndTruth.com
# https://www.SpiritAndTruth.org
#
#
#############################################################################
VERSION = 0.4

HISTORY = '''

    0.4 Corrected transliteration of BSTGreek character '&' (combination
        rough breathing mark with acute). This was not being transliterated 
        by an accented "h."

    0.3 Added translitToUnicodeRtf() function. Added closing ';' to
        HTML unicode values. Corrected translitBSTGreek() to omit smooth
        breath mark from transliterations.

    0.2 Fixed Non-ASCII character embedded in comment which generated a
        warning.
'''

HEBREW_LETTERS = {

    r'alep'          : r'{t %146a_l&p_}',
    r'atenah'        : r'{t %146at_na_h.}',
    r'atnah'         : r'{t %146at_na_h.}',
    r'ayin'          : r'{t %145ayin}',
    r'begad'         : r'{t be_g_ad_}',
    r'bet'           : r'{t be^t_}',
    r'dages'         : r'{t da_g_e_s~}',
    r'dalet'         : r'{t da_l&t_}',
    r'darga'         : r'{t darga_%146}',
    r'geres'         : r'{t g&r&s~}',
    r'gimel'         : r'{t gi^m&l}',
    r'hatep'         : r'{t h.a_t_e_p_}',
    r'hatup'         : r'{t h.a_t.u^p_}',
    r'het'           : r'{t h.e^t_}',
    r'he'            : r'{t he_%146}',
    r'hifil'         : r'{t hip_%145i^l}',
    r'hipil'         : r'{t hip_%145i^l}',
    r'hiqtil'        : r'{t hiqt.i^l}',
    r'hireq'         : r'{t h.i^r&q}',
    r'histafel'      : r'{t his~tap_e_l}',
    r'histapel'      : r'{t his~tap_e_l}',
    r'hitpael'       : r'{t hit_pa%145e_l}',
    r'hitpolel'      : r'{t hit_po^le_l}',
    r'hitqattel'     : r'{t hit_qat.t.e_l}',
    r'hofal'         : r'{t hop_%145al}',
    r'holem'         : r'{t h.o^l&m}',
    r'hopal'         : r'{t hop_%145al}',
    r'hoqtal'        : r'{t hiqt.al}',
    r'kap'           : r'{t kap_}',
    r'kepat'         : r'{t ke_p_at_}',
    r'lamed'         : r'{t la_m&d_}',
    r'legarma'       : r'{t le+g_arm&h}',
    r'mahpak'        : r'{t mahpak_}',
    r'mappiq'        : r'{t mappi^q}',
    r'maqqep'        : r'{t ma_qqe_p_}',
    r'mem'           : r'{t me_m}',
    r'mereka'        : r'{t me^re+k_a_%146}',
    r'merekha'       : r'{t me^re+k_a_%146}',
    r'meteg'         : r'{t m&t_&g_}',
    r'munah'         : r'{t mu^nah.}',
    r'nifal'         : r'{t nip_%145al}',
    r'nipal'         : r'{t nip_%145al}',
    r'niqtal'        : r'{t niqt.al}',
    r'nun'           : r'{t nu^n}',
    r'pasta'         : r'{t pas~t.a_%146}',
    r'pasuq'         : r'{t pa_su^q}',
    r'patah'         : r'{t pat_ah.}',
    r'pazer'         : r'{t pa_ze_r}',
    r'pe'            : r'{t pe%146}',
    r'piel'          : r'{t pi%145e_l}',
    r'polal'         : r'{t po^lal}',
    r'polel'         : r'{t po^le_l}',
    r'pual'          : r'{t pu%145al}',
    r'qal'           : r'{t qal}',
    r'qames'         : r'{t qa_me_s.}',
    r'qatal'         : r'{t qa_t.al}',
    r'qatel'         : r'{t qa_t.e_l}',
    r'qatil'         : r'{t qa_t.i^l}',
    r'qatlal'        : r'{t qat.la_l}',
    r'qatol'         : r'{t qa_t.o^l}',
    r'qato_l'        : r'{t qa_t.o_l}',
    r'qattal'        : r'{t qat.t.a_l}',
    r'qattil'        : r'{t qat.t.i^l}',
    r'qatul'         : r'{t qa_t.u^l}',
    r'qatu_l'        : r'{t qa_t.u_l}',
    r'qetala'        : r'{t q&t.a_la^}',
    r'qetaltal'      : r'{t q&t.alta_l}',
    r'qetulla'       : r'{t q&t.u_lla^}',
    r'qibbus'        : r'{t qibbu^s.}',
    r'qittel'        : r'{t qit.t.e_l}',
    r'qop'           : r'{t qo^p_}',
    r'qotel'         : r'{t qo^t.e_l}',
    r'qtl'           : r'{t qt.l}',
    r'quttal'        : r'{t qut.t.al}',
    r'rebia'         : r'{t re+bi^a%145}',
    r'res'           : r'{t re^s~}',
    r'sade'          : r'{t s.a_d_e^}',
    r'samek'         : r'{t sa_m&k_}',
    r'segol'         : r'{t s&g_o^l}',
    r'segolta'       : r'{t se+g_o^lta_%146}',
    r'sere'          : r'{t s.e_re^}',
    r'sewa'          : r'{t s~e+wa_%146}',
    r'shin'          : r'{t s~i^n}',
    r'silluq'        : r'{t sillu^q}',
    r'sin'           : r"{t s'i^n}",
    r'sop_pasuq'     : r'{t sop_ pa_su^q}',
    r'sureq'         : r'{t s~u^r&q}',
    r'tav'           : r'{t ta_w}',
    r'taw'           : r'{t ta_w}',
    r'tebir'         : r'{t te+b_i^r}',
    r'telisa_gedola' : r'{t te+li^s~a_%146 ge+do^la_h}',
    r'tet'           : r'{t t.e^t_}',
    r'tevir'         : r'{t te+b_i^r}',
    r'tipa'          : r'{t t.ip_h.a_h}',
    r'tipha'         : r'{t t.ip_h.a_h}',
    r'vav'           : r'{t wa_w}',
    r'waw'           : r'{t wa_w}',
    r'wayyiqtol'     : r'{t wayyiqt.ol}',
    r'wayyqtl'       : r'{t wayyqt.l}',
    r'weqatal'       : r'{t w&qat.al}',
    r'weyiqtol'      : r'{t w&yiqt.ol}',
    r'yaqtul'        : r'{t yaqt.ul}',
    r'yaqtulu'       : r'{t yaqt.ulu}',
    r'yetib'         : r'{t ye+t_i^b_}',
    r'yetiv'         : r'{t ye+t_i^b_}',
    r'yiqtol'        : r'{t yiqt.ol}',
    r'yqtl'          : r'{t yqt.l}',
    r'yod'           : r'{t yo^d_}',
    r'zaqep_gadol'   : r'{t za_qe_p_ ga_d_o^l}',
    r'zaqep_qaton'   : r'{t za_qe_p_ qa_t.o_n}',
    r'zarqa'         : r'{t zarqa_%146}',
    r'zayin'         : r'{t zayin}',

    }

    
##############################################################
# expand_keywords() : identify and expand Hebrew grammar names
##############################################################
wordPat = r'{(?P<text>[^}]+)}'
wordExp = re.compile( wordPat )
def expand_keywords( text ) :
    # Expand Hebrew punctuation keywords into transliteration mappings
    output = ''
    remainder = text
    while 1 :
        match = wordExp.search( remainder )
        if match :
            output += remainder[ 0 : match.start() ]
            word = match.group( 'text' )
            if HEBREW_LETTERS.has_key( word ) :
                output += HEBREW_LETTERS[ word ]
            else :
                output += match.group()
            remainder = remainder[ match.end() : ]
        else :
            break
    return output + remainder

###################################################################
# finals() : Find and convert final characters for Hebrew and Greek
###################################################################
gPat = r'{g (?P<text>[^}]+)}'
gRE = re.compile( gPat )
hPat = r'{h (?P<text>[^}]+)}'
hRE = re.compile( hPat )
def finals( line ) :

    old = line

    # Convert Greek finals.
    output = ''
    remainder = line
    while 1 :
        match = gRE.search( remainder )
        if match :

            # pass through text before Greek, isolate remainder
            output = output + remainder[ 0 : match.start( 'text' ) ]
            remainder = remainder[ match.end( 'text' ) : ]

            # convert the Greek
            output = output + greekConvertFinals( match.group( 'text' ) )

        else :
            break

    line = output + remainder

    # Convert Hebrew finals.
    output = ''
    remainder = line
    while 1 :
        match = hRE.search( remainder )
        if match :

            # pass through text before Hebrew, isolate remainder
            output = output + remainder[ 0 : match.start( 'text' ) ]
            remainder = remainder[ match.end( 'text' ) : ]

            # convert the Hebrew
            output = output + hebrewConvertFinals( match.group( 'text' ) )

        else :
            break

    line = output + remainder
    return line

def hebrewConvertFinals( text ) :
    out = text
    out = re.sub( r'(\S)k(\s|$)', r'\1$\2', out ) #kap
    out = re.sub( r'(\S)m(\s|$)', r'\1~\2', out ) #mem
    out = re.sub( r'(\S)n(\s|$)', r'\1!\2', out ) #nun
    out = re.sub( r'(\S)p(\s|$)', r'\1@\2', out ) #pe
    out = re.sub( r'(\S)c(\s|$)', r'\1#\2', out ) #sade
    return out

def greekConvertFinals( text ) :
    out = text
    out = re.sub( r'(\S)s(\s|$)', r'\1"\2', out ) #sigma
    return out

BSTGREEK_TO_UNICODE_GREEK = {
# See https://www.alanwood.net/unicode/greek.html

    ' '  : ' ', # space

    # Cheating - borrowed breath marks from "Spacing Modified Letters"
    'j'  : '&#700;',
    'J'  : '&#701;',

    # Accent marks
    'v'  : '&#714;',
    '#'  : '&#714;',
    'V'  : '&#714;',

    '"'  : '&#962;',
    '>'  : None,        # FLAG ERROR IF USED
    '{'  : None,        # ERROR
    '}'  : None,        # ERROR

    'A'  : '&#913;',
    'B'  : '&#914;',
    'G'  : '&#915;',
    'D'  : '&#916;',
    'E'  : '&#917;',
    'Z'  : '&#918;',
    'H'  : '&#919;',
    'Q'  : '&#920;',
    'I'  : '&#921;',
    'K'  : '&#922;',
    'L'  : '&#923;',
    'M'  : '&#924;',
    'N'  : '&#925;',
    'X'  : '&#926;',
    'O'  : '&#927;',
    'P'  : '&#928;',
    'R'  : '&#929;',
    'S'  : '&#931;',
    'T'  : '&#932;',
    'U'  : '&#933;',
    'F'  : '&#934;',
    'C'  : '&#935;',
    'Y'  : '&#936;',
    'W'  : '&#937;',

    'a'  : '&#945;',
    'b'  : '&#946;',
    'g'  : '&#947;',
    'd'  : '&#948;',
    'e'  : '&#949;',
    'z'  : '&#950;',
    'h'  : '&#951;',
    'q'  : '&#952;',
    'i'  : '&#953;',
    'k'  : '&#954;',
    'l'  : '&#955;',
    'm'  : '&#956;',
    'n'  : '&#957;',
    'x'  : '&#958;',
    'o'  : '&#959;',
    'p'  : '&#960;',
    'r'  : '&#961;',
    's'  : '&#963;',
    't'  : '&#964;',
    'u'  : '&#965;',
    'f'  : '&#966;',
    'c'  : '&#967;',
    'y'  : '&#968;',
    'w'  : '&#969;',

    'I>'  : '&#938;',
    'Y>'  : '&#939;',
    'av'  : '&#940;',
    'aV'  : '&#940;',
    'ev'  : '&#941;',
    'eV'  : '&#941;',
    'hv'  : '&#942;',
    'hV'  : '&#942;',
    'iv'  : '&#943;',
    'iV'  : '&#943;',
    'u?'  : '&#944;',


    'v>'   : '&#901;',
    'i>v'  : '&#912;',
    'u>v'  : '&#944;',
    'i>'   : '&#970;',
    'u>'   : '&#971;',
    }
BSTGREEK_TO_UNICODE_GREEK_EXTENDED = {
# See https://www.alanwood.net/unicode/greek_extended.html
    '~'  : '&#8142;',
    '`'  : '&#8175;',
    '^'  : '&#8157;',
    '&'  : '&#8158;',
    '|'  : '&#8159;',
    '\\' : '&#8143;',
    ';'  : '&#8175;',
    "'"  : '&#8128;',
    '?'  : '&#8174;',
    '/'  : '&#8126;',
    '['  : '&#8142;',
    ']'  : '&#8141;',

    'aj'  : '&#7936;',
    'aJ'  : '&#7937;',
    'a['  : '&#7938;',
    'a\^'  : '&#7939;',
    'a~'  : '&#7940;',
    'a&'  : '&#7941;',
    'a\\' : '&#7942;',
    'a|'  : '&#7943;',

    'Aj'  : '&#7944;',
    'AJ'  : '&#7945;',
    'A['  : '&#7946;',
    'A\^'  : '&#7947;',
    'A~'  : '&#7948;',
    'A&'  : '&#7949;',
    'A\\' : '&#7950;',
    'A|'  : '&#7951;',

    'ej'  : '&#7952;',
    'eJ'  : '&#7953;',
    'e]'  : '&#7954;',
    'e\^'  : '&#7955;',
    'e~'  : '&#7956;',
    'e&'  : '&#7957;',

    'Ej'  : '&#7960;',
    'EJ'  : '&#7961;',
    'E]'  : '&#7962;',
    'E\^'  : '&#7963;',
    'E~'  : '&#7964;',
    'E&'  : '&#7965;',

    'hj'  : '&#7968;',
    'hJ'  : '&#7969;',
    'h['  : '&#7970;',
    'h\^'  : '&#7971;',
    'h~'  : '&#7972;',
    'h&'  : '&#7973;',
    'h\\' : '&#7974;',
    'h|'  : '&#7975;',

    'Hj'  : '&#7976;',
    'HJ'  : '&#7977;',
    'H['  : '&#7978;',
    'H\^'  : '&#7979;',
    'H~'  : '&#7980;',
    'H&'  : '&#7981;',
    'H\\' : '&#7982;',
    'H|'  : '&#7983;',

    'ij'  : '&#7984;',
    'iJ'  : '&#7985;',
    'i['  : '&#7986;',
    'i\^'  : '&#7987;',
    'i~'  : '&#7988;',
    'i&'  : '&#7989;',
    'i\\' : '&#7990;',
    'i|'  : '&#7991;',

    'Ij'  : '&#7992;',
    'IJ'  : '&#7993;',
    'I['  : '&#7994;',
    'I\^'  : '&#7995;',
    'I~'  : '&#7996;',
    'I&'  : '&#7997;',
    'I\\' : '&#7998;',
    'I|'  : '&#7999;',

    'oj'  : '&#8000;',
    'oJ'  : '&#8001;',
    'o]'  : '&#8002;',
    'o\^'  : '&#8003;',
    'o~'  : '&#8004;',
    'o&'  : '&#8005;',

    'Oj'  : '&#8008;',
    'OJ'  : '&#8009;',
    'O]'  : '&#8010;',
    'O\^'  : '&#8011;',
    'O~'  : '&#8012;',
    'O&'  : '&#8013;',

    'uj'  : '&#8016;',
    'uJ'  : '&#8017;',
    'u['  : '&#8018;',
    'u\^'  : '&#8019;',
    'u~'  : '&#8020;',
    'u&'  : '&#8021;',
    'u\\' : '&#8022;',
    'u|'  : '&#8023;',

    'Yj'  : '&#8025;',
    'Y\^'  : '&#8027;',
    'Y&'  : '&#8029;',
    'Y|'  : '&#8031;',

    'wj'  : '&#8032;',
    'wJ'  : '&#8033;',
    'w['  : '&#8034;',
    'w\^'  : '&#8035;',
    'w~'  : '&#8036;',
    'w&'  : '&#8037;',
    'w\\' : '&#8038;',
    'w|'  : '&#8039;',

    'Wj'  : '&#8040;',
    'WJ'  : '&#8041;',
    'W['  : '&#8042;',
    'W\^'  : '&#8043;',
    'W~'  : '&#8044;',
    'W&'  : '&#8045;',
    'W\\' : '&#8046;',
    'W|'  : '&#8047;',

    'a;'  : '&#8048;',
    'av'  : '&#8049;',

    'e;'  : '&#8050;',
    'ev'  : '&#8051;',

    'h;'  : '&#8052;',
    'hv'  : '&#8053;',

    'i;'  : '&#8054;',
    'iv'  : '&#8055;',

    'o;'  : '&#8056;',
    'ov'  : '&#8057;',

    'u;'  : '&#8058;',
    'uv'  : '&#8059;',

    'w;'  : '&#8060;',
    'wv'  : '&#8061;',

    'aj/'  : '&#8064;',
    'aJ/'  : '&#8065;',
    'a]/'  : '&#8066;',
    'a\^/'  : '&#8067;',
    'a~/'  : '&#8068;',
    'a&/'  : '&#8069;',
    'a\\/' : '&#8070;',
    'a|/'  : '&#8071;',

    'Aj/'  : '&#8072;',
    'AJ/'  : '&#8073;',
    'A]/'  : '&#8074;',
    'A\^/'  : '&#8075;',
    'A~/'  : '&#8076;',
    'A&/'  : '&#8077;',
    'A\\/' : '&#8078;',
    'A|/'  : '&#8079;',

    'hj/'  : '&#8080;',
    'hJ/'  : '&#8081;',
    'h]/'  : '&#8082;',
    'h\^/'  : '&#8083;',
    'h~/'  : '&#8084;',
    'h&/'  : '&#8085;',
    'h\\/' : '&#8086;',
    'h|/'  : '&#8087;',

    'Hj/'  : '&#8088;',
    'HJ/'  : '&#8089;',
    'H]/'  : '&#8090;',
    'H\^/'  : '&#8091;',
    'H~/'  : '&#8092;',
    'H&/'  : '&#8093;',
    'H\\/' : '&#8094;',
    'H|/'  : '&#8095;',

    'wj/'  : '&#8096;',
    'wJ/'  : '&#8097;',
    'w]/'  : '&#8098;',
    'w\^/'  : '&#8099;',
    'w~/'  : '&#8100;',
    'w&/'  : '&#8101;',
    'w\\/' : '&#8102;',
    'w|/'  : '&#8103;',

    'Wj/'  : '&#8104;',
    'WJ/'  : '&#8105;',
    'W]/'  : '&#8106;',
    'W\^/'  : '&#8107;',
    'W~/'  : '&#8108;',
    'W&/'  : '&#8109;',
    'W\\/' : '&#8110;',
    'W|/'  : '&#8111;',

    'a;/'  : '&#8114;',
    'a/'   : '&#8115;',
    'av/'  : '&#8116;',
    "a`"   : '&#8118;',
    "a`/"  : '&#8119;',

    "A;"   : '&#8122;',
    "Av"   : '&#8123;',
    "A/"   : '&#8124;',

    'h;/'  : '&#8130;',
    'h/'   : '&#8131;',
    'hv/'  : '&#8132;',
    "h`"   : '&#8134;',
    "h`/"  : '&#8135;',

    "E;"   : '&#8136;',
    "Ev"   : '&#8137;',

    "H;"   : '&#8138;',
    "Hv"   : '&#8139;',
    "H/"   : '&#8140;',

    'i>;'  : '&#8146;',
    'iv>'  : '&#8147;',
    "i'"   : '&#8150;',
    "i'/"  : '&#8151;',

    "I;"   : '&#8154;',
    "Iv"   : '&#8155;',

    'u;>'  : '&#8162;',
    'uv>'  : '&#8163;',

    'rj'   : '&#8164;',
    'rJ'   : '&#8165;',

    "u'"   : '&#8166;',
    "u'>"  : '&#8167;',

    'w;/'  : '&#8178;',
    'w/'   : '&#8179;',
    'wvi'  : '&#8180;',
    "w'"   : '&#8182;',
    "w'/"  : '&#8183;',

    'O;'   : '&#8184;',
    'Ov'   : '&#8185;',
    'W;'   : '&#8186;',
    'Wv'   : '&#8187;',
    'W/'   : '&#8188;',

    }

BSTHEBREW_TO_UNICODE_HEBREW = {
# See https://www.alanwood.net/unicode/hebrew.html

    ' '  : ' ', # space

    '>'  : '&#1456;', # shewa
    '.'  : '&#1456;',
    '?'  : '&#1457;', # hateph segol
    '/'  : '&#1457;',
    '}'  : '&#1458;', # hateph patach
    ']'  : '&#1458;',
    '\\' : '&#1459;', # hateph qamets
    'I'  : '&#1460;', # hireq
    'i'  : '&#1460;', # hireq
    'E'  : '&#1461;', # tsere
    'e'  : '&#1461;', # tsere
    ','  : '&#1462;', # segol
    ':'  : '&#1463;', # patach
    ';'  : '&#1463;',
    '"'  : '&#1464;', # qametz
    "'"  : '&#1464;',
    'O'  : '&#1465;', # holem
    'o'  : '&#1465;', # holem
    'U'  : '&#1467;', # qibbutz
    'u'  : '&#1467;', # qibbutz
    '{'  : '&#1468;', # dagesh, mappiq
    '*'  : '&#1469;', # metheg
    '('  : '&#1469;', # metheg
    ')'  : '&#1469;', # metheg
    '-'  : '&#1479;', # maqqeph
    '`'  : '&#1475;', # sof pasuq

    '%'  : '&#1498;&#1475;', # final kaf, sof pasuq
    '\^'  : '&#1498;&#1464;', # final kaf, qametz
    '&'  : '&#1498;&#1468;', # final kaf, mappiq

    'A'  : '&#1468;&#1493;',  # holem waw
    'B'  : '&#1489;&#1468;',  # bet dagesh
    'G'  : '&#1490;&#1468;',  # gimmel dagesh
    'D'  : '&#1491;&#1468;',  # dalet dagesh
    'H'  : '&#1492;&#1468;',  # he dagesh
    'W'  : '&#1493;&#1468;',  # waw dagesh
    'Z'  : '&#1494;&#1468;',  # zayin dagesh
    'X'  : '&#1495;&#1468;',  # het dagesh
    'J'  : '&#1496;&#1468;',  # tet dagesh
    'Y'  : '&#1497;&#1468;',  # yod dagesh
    'K'  : '&#1499;&#1468;',  # kaph dagesh
    'L'  : '&#1500;&#1468;',  # lamed dagesh
    'M'  : '&#1502;&#1468;',  # mem dagesh
    'N'  : '&#1504;&#1468;',  # nun dagesh
    'S'  : '&#1505;&#1468;',  # samek dagesh
    'P'  : '&#1508;&#1468;',  # pe dagesh
    'C'  : '&#1510;&#1468;',  # tsadi dagesh
    'Q'  : '&#1511;&#1468;',  # qoph dagesh
    'R'  : '&#1512;&#1468;',  # resh dagesh
    'F'  : '&#1513;&#1474;',  # shin = sin, dot
    'V'  : '&#1513;&#1473;',  # shin = sin, dot
    'T'  : '&#1514;&#1468;',  # taw dagesh

    'a'  : '&#1488;', # aleph
    'b'  : '&#1489;',  # bet
    'g'  : '&#1490;',  # gimmel
    'd'  : '&#1491;',  # dalet
    'h'  : '&#1492;',  # he
    'w'  : '&#1493;',  # waw
    'z'  : '&#1494;',  # zayin
    'x'  : '&#1495;',  # het
    'j'  : '&#1496;',  # tet
    'y'  : '&#1497;',  # yod
    '$'  : '&#1498;',  # kaph final
    'k'  : '&#1499;',  # kaph
    'l'  : '&#1500;',  # lamed
    '~'  : '&#1501;',  # mem final
    'm'  : '&#1502;',  # mem
    '!'  : '&#1503;',  # nun final
    'n'  : '&#1504;',  # nun
    's'  : '&#1505;',  # samek
    '['  : '&#1506;',  # ayin
    '@'  : '&#1507;',  # pe final
    'p'  : '&#1508;',  # pe
    '#'  : '&#1509;',  # tsadi final
    'c'  : '&#1510;',  # tsadi
    'q'  : '&#1511;',  # qoph
    'r'  : '&#1512;',  # resh
    'f'  : '&#1513;&#1474;', # shin = sin, dot
    'v'  : '&#1513;&#1473;', # shin = sin, dot
    't'  : '&#1514;',  # taw

    'ww'  : '&#1520;', # wav-wav
    'wy'  : '&#1521;', # wav-yod
    'yy'  : '&#1522;', # yod-yod

    }

BSTHEBREW_TO_UNICODE_ALPHABETIC = {
# See https://www.alanwood.net/unicode/alphabetic_presentation_forms.html
#
# Browsers are inconsistent in how they handle Hebrew glyphs from this
# code area. Firefox treats them as legitimate Hebrew (displays them
# in right-to-left order) whereas IE does not (displays them as mere symbols
# in default left-to-right order). Therefore, we don't recommend using
# these mappings.
#
# We originally thought these might produce nicer looking Hebrew
# due to some of the combined characters, but the inconsistencies in how
# they are handled are too limiting.
#

    'A'  : '&#64331;', # holem waw
    
    'B'  : '&#64305;', # bet dagesh
    'G'  : '&#64306;', # gimel dagesh
    'D'  : '&#64307;', # dalet dagesh
    'H'  : '&#64308;', # he dagesh
    'W'  : '&#64309;', # waw dagesh
    'Z'  : '&#64310;', # zayin dagesh
    'X'  : '&#1513;', # shin (featureless)
    'J'  : '&#64312;', # tet dagesh
    'Y'  : '&#64313;', # yod dagesh
    '&'  : '&#64309;', # kaph final dagesh
    'K'  : '&#64315;', # kaph dagesh
    'L'  : '&#64316;', # lamed dagesh
    'M'  : '&#64318;', # mem dagesh
    'N'  : '&#64320;', # nun dagesh
    'S'  : '&#64321;', # samek dagesh
    'P'  : '&#64324;', # pe dagesh
    '@{' : '&#64323;', # pe final with dagesh
    'C'  : '&#64326;', # tsade dagesh
    'Q'  : '&#64327;', # qoph dagesh
    'R'  : '&#64328;', # resh dagesh
    'T'  : '&#64330;', # taw dagesh
    'V'  : '&#64300;', # shin dagesh
    'F'  : '&#64301;', # sin dagesh

    'a;' : '&#64302;', # aleph patach
    'a:' : '&#64302;', # aleph patach
    'a"' : '&#64303;', # aleph qametz
    "a'" : '&#64303;', # aleph qametz
    'a{' : '&#64304;', # aleph dagesh

    'v'  : '&#64298;', # shin
    'f'  : '&#64299;', # sin

    }

############################################################################
# Translate Text using Specified Dictionary(s)
############################################################################
def DictionaryTranslate( dicts, text, reverse = 0 ) :
    # Default does not reverse character order.

    # While more text left to translate.
    unicode = ''
    maxkeylength = 3
    i = 0
    while ( i < len( text ) ) :

        # Compute longest possible input match available.
        out = ''
        if len( text ) < i + maxkeylength : maxkeylength = len( text ) - i

        # For longest match to shortest match.
        for length in range( maxkeylength, 0, -1 ) :

            # For each dictionary being used.
            key = text[ i : i + length ]
            #print 'FOO LENGTH&KEY', length, key
            for dict in dicts :

                # If dictionary has this piece of text as a key.
                if dict.has_key( key ) :

                    # Translate to the corresponding output.
                    out = dict[ key ]
                    #print 'TRANSLATE', key, 'to', out

                    # If valid, then we are done with this piece.
                    if out : break

                    # If we get here, then we've tried to translate an
                    # unsupported sequence of text.
                    print 'UNSUPPORTED TRANSLATION ERROR:', text
                    print 'TRANSLATION KEY:', key
                    print 'TRANSLATION DICT:', dict
                    assert 0

                # Else not found in dictionary.
                else :

                    # If key is minimum length.
                    if 1 == len( key ) :

                        # Pass through input untranslated.
                        # (This code is the same in source and destination.)
                        out = key

            # If we found a matching output, we are done with this text
            # chunk.
            if out : break

        # Reverse the results, if requested.
        if reverse :
            unicode = out + unicode
        else :
            unicode = unicode + out

        # Move to next chunk of text to be translated.
        i += length

    return unicode

############################################################################
# Translate from BSTHebrew to Unicode HTML
############################################################################
def BSTHebrewToUnicodeHtml( text,  extended_codes = 0, reverse = 1 ) :
    # Assumes incoming text is arranged left-to-right and
    # returns right-to-left translation by default.
    dicts = []
    if extended_codes : dicts.append( BSTHEBREW_TO_UNICODE_ALPHABETIC )
    dicts.append( BSTHEBREW_TO_UNICODE_HEBREW )
    out = DictionaryTranslate( dicts, text, reverse )
    if not reverse :
        parts = out.split( '&' )
        parts.reverse
        out = string.join( parts, '&' )
    return out

############################################################################
# Translate from BSTGreek to Unicode HTML
############################################################################
def BSTGreekToUnicodeHtml( text, extended_codes = 0 ) :
    dicts = []
    if extended_codes : dicts.append( BSTGREEK_TO_UNICODE_GREEK_EXTENDED )
    dicts.append( BSTGREEK_TO_UNICODE_GREEK )
    return DictionaryTranslate( dicts, text )


############################################################################
# Translate from HTML to RTF Unicode
############################################################################
def htmlToRtfUnicode( text  ) :
    text = re.sub( '<sup>([^>]+)</sup>', r'{\super \1}', text )
    text = re.sub( '&#(\d+);', r'\u\1?', text )
    return text


############################################################################
# Advance the Breath Mark for Capital Letters
############################################################################
def advanceBreathMarkCapital( text, withSpace = 1 ) :
    # Move breath mark following initial capital letter.
    replace = r'\2\1'
    if withSpace : replace = ' ' + replace

    # Single vowels (insert space before breath mark when it is advanced?).
    text = re.sub( r'([AEHIORWU])([jJ&^|])', replace, text )

    return text

############################################################################
# Advance the Breath Mark for Capital Letters
############################################################################
def advanceBreathMark( text, withSpace = 1 ) :
    # Move breath mark.
    replace = r'\2\1'
    if withSpace : replace = ' ' + replace

    # Diphthongs
    text = re.sub( r'(ai)([j&^|])(?i)', replace, text )
    text = re.sub( r'(ei)([j&^|])(?i)', replace, text )
    text = re.sub( r'(oi)([j&^|])(?i)', replace, text )
    text = re.sub( r'(ui)([j&^|])(?i)', replace, text )
    text = re.sub( r'(au)([j&^|])(?i)', replace, text )
    text = re.sub( r'(eu)([j&^|])(?i)', replace, text )
    text = re.sub( r'(ou)([j&^|])(?i)', replace, text )

    # Single vowels
    text = re.sub( r'([aehiorwu])([j&^])(?i)', replace, text )


    return text

############################################################################
# Transliterate BSTGreek into corresponding ASCII characters.
############################################################################
BSTGREEK_ACCENTS = [ '~', '`', '#', '^', '&', '|', '\\', '\;', "'", '>', '?', '[', ']', 'V', 'v' ]

def translitBSTGreek( text ) :
    translit = text
    # Produce automatic transliteration.
    # If we have at least a two character word.
    charCount = 0
    for code in translit :
        if not BSTGREEK_ACCENTS.count( code ) :
            charCount += 1
        if 2 <= charCount : break
    if charCount < 2 :
        translit = ''
    else :

        # Expand combination breath/accent marks.
        input = translit
        translit = re.sub( '\|',    r"J'", translit )
        translit = re.sub( r'\\',    r"j'", translit )
        translit = re.sub( '&',     r'Jv', translit )
        translit = re.sub( '[\[~]', r'jv', translit )
        translit = re.sub( '\^',    r'J;', translit )
        translit = re.sub( '\]',    r'j;', translit )

        # Move the breath mark forward.
        translit = advanceBreathMarkCapital( translit )
        translit = advanceBreathMark( translit )

        # Nasal gamma.
        translit = re.sub( r'(g)([cgkx])(?i)', r'n\2', translit )

        # Mappings from "Dictionary of Premillennial Theology," p. 21.
        translit = re.sub( r'(c)', r'c1', translit ) #1 = h
        translit = re.sub( r'(C)', r'C1', translit ) #1 = h
        translit = re.sub( r'(f)', r'p1', translit ) #1 = h
        translit = re.sub( r'(F)', r'P1', translit ) #1 = h
        translit = re.sub( r'(q)', r't1', translit ) #1 = h
        translit = re.sub( r'(Q)', r'T1', translit ) #1 = h
        #translit = re.sub( r'(x)(?i)', r'ch', translit )
        translit = re.sub( r'(y)', r'ps', translit ) #Dictionary has 'ch' here [ERROR]?
        translit = re.sub( r'(Y)', r'Ps', translit ) #Dictionary has 'ch' here [ERROR]?

        # Find and convert 'h' before we add it in subsequent transliterations.
        translit = re.sub( r'(w)', r'o-', translit )
        translit = re.sub( r'(W)', r'O-', translit )
        translit = re.sub( r'(h)', r'e-', translit )
        translit = re.sub( r'(H)', r'E-', translit )

        # remove dialytica
        translit = re.sub( r'([aeiou])>', r'\1', translit )

        # Special case when upsilon is alone as a vowel.
        translit = re.sub( r"([^AEIOUaeiou])(u)([^AEIOUaeiou'` $])", r'\1y\3', translit )
        translit = re.sub( r"([^AEIOUaeiou])(U)([^AEIOUaeiou'` $])", r'\1Y\3', translit )

        # Handle accent?
        translit = re.sub( r'[Vv\\\[;]', r"'", translit )

        # Convert other markups.
        translit = re.sub( r'(")', r"s", translit )
        #translit = re.sub( r'(/)', r"i", translit )
        translit = re.sub( r'(/)', r"", translit )
        translit = re.sub( r'(\[\\;)', r'', translit )

        # Restore our added 'h' which was mapped to '1' to avoid subsequent expansion.
        translit = re.sub( r'1', r'h', translit ) #1 = h

        # Handle breath marks next.
        # Expunge non-vocal breath marks.
        # (May have leading space if breath mark was advanced.)
        translit = re.sub( '(\s+)j', r'\1', translit )

        # Breath mark begins the word.
        result = ''
        remainder = translit
        translit = ''
        while 1:
            match = re.match( '^\s*(?P<word>[J&^|]\w)', remainder )
            if match :

                # Convert breath mark, uncapitalize whatever follows.
                matchText = match.group( 'word' )
                jIndex = match.start( 'word' )
                next = matchText[ 1 ]
                breath = 'H'
                if 'a' <= next and next <= 'z' : breath = 'h'
                translit += remainder[ 0 : jIndex ]
                translit += breath 
                translit += matchText[ 1 : ].lower()

                remainder = remainder[ match.end( 'word' ) : ]

                #print 'FOO TRANSLIT', translit
                #print 'FOO REMAINDER', remainder

            else :
                break

        translit += remainder

        translit = re.sub( r'([J&^|])', r'h', translit )
        translit = re.sub( r'^\s+h', r'h', translit )

        # Convert accent to emboldened.
        # Accent includes final.
        if 0 :
            test = translit
            #translit = re.sub( r"(([^aeiouwy ]|[ch)?)(ai|ei|ui|oi|au|eu|ou|[aeiouwy])'([^aeiouw])\b(?i)", self.boldOpen() + r'\1\3\4' + self.boldClose(), translit )
            translit = re.sub( r"(([^aeiouwy> ]|[cpt]h)?)(ai|ei|ui|oi|au|eu|ou|[aeiouwy])'([^aeiouw])\b(?i)", self.boldOpen() + r'\1\3\4' + self.boldClose(), translit )
            # Accent within word.
            #translit = re.sub( "(([^aeiouwy ]|ch)?)(ai|ei|ui|oi|au|eu|ou|[aeiouwy])'([^aeiouwy ]?)(?![aeiouwyh ])(?i)", self.boldOpen() + r'\1\3\4' + self.boldClose(), translit )
            translit = re.sub( "(([^aeiouwy> ]|[cpt]h)?)(ai|ei|ui|oi|au|eu|ou|[aeiouwy])'([^aeiouwy ]?)(?![aeiouwyh ])(?i)", self.boldOpen() + r'\1\3\4' + self.boldClose(), translit )
            # Left overs not matched above (e.g. kardi'as)
            translit = re.sub( "(([^aeiouwy ]|[cpt]h)?)(ai|ei|ui|oi|au|eu|ou|[aeiouwy])'(?i)", self.boldOpen() + r'\1\3' + self.boldClose(), translit )
            if translit != test : print 'FOOTEST', test, ' -> ', translit
        else :
            translit = re.sub( r"'", "", translit )

        if 0 and translit != input :
            print 'TRANSLIT IN ', input
            print 'TRANSLIT OUT', translit

    return translit

############################################################################
# Transliterate BSTHebrew into corresponding ASCII characters.
############################################################################
BSTHEBREW_SHORT_VOWELS = [ 
    ';',   # patah
    ',',   # segol
    'i',   # hireq
    "'",   # qames
    'u',   # qibbus
    ]

BSTHEBREW_SHEWAS = [ ';', '.' ]

def translitBSTHebrew( text ) :

    # Mappings from "Dictionary of Premillennial Theology," p. 21.
    translit = ''
    if 1 < len( text ) :
        translit = text
        input = text

        # Single shewa (follows short vowel or first of two consecutive shewas
        # or end of word). See "Basics of Biblical Hebrew" by Pratico and
        # Van Pelt, p. 20
        short = -1
        output = ''
        shewas = 0
        for i in range( len( translit )  ):
            code = translit[ i ]

            # If this is a shewa.
            if BSTHEBREW_SHEWAS.count( code ) :

                # If at end of word, it is silent and we skip it.
                if i + 1 == len( translit ) : continue

                # If the next vowel is a shewa, this one is silent so we skip it.
                if i + 2 < len( translit ) and BSTHEBREW_SHEWAS.count( translit[ i + 2 ] ) : continue

                # If the previous vowel is short, this is silent and we skip it.
                if -1 != short and i - 2 == short : continue

            # Keep track of short vowels as we move along
            if BSTHEBREW_SHORT_VOWELS.count( code ) : 
                maybeShort = i
                # Not short if followed by yod (indicates unchangeable long).
                if i + 1 < len( translit ) and 'y' == translit[ i + 1 ] : maybeShort = -1
                if -1 != maybeShort : short = maybeShort

            # Pass code through, it isn't a silent shewa.
            output += code

        translit = output

        # Finals
        translit = re.sub( r'~' , r"m"  , translit )
        translit = re.sub( r'!' , r"n"  , translit )
        translit = re.sub( r'@' , r"p"  , translit )
        translit = re.sub( r'#' , r"s2" , translit ) # 2 = .
        translit = re.sub( r'\$', r"k"  , translit )
        translit = re.sub( r'%' , r"k"  , translit )
        translit = re.sub( r'\^', r"ka" , translit )
        translit = re.sub( r'&' , r"k"  , translit )

        # Consonants
        translit = re.sub( r'\['   , r"3"      , translit ) # 3 = '
        translit = re.sub( r'\bA'  , r"12"     , translit ) # 12 = wo-
        translit = re.sub( r'A'    , r"o"      , translit )
        translit = re.sub( r'a'    , r"1"      , translit ) # 1 = `
        translit = re.sub( r'(\S)B', r"\1bb"   , translit )
        translit = re.sub( r'C'    , r"S2"     , translit ) # 2 = .
        translit = re.sub( r'c'    , r"s2"     , translit ) # 2 = .
        translit = re.sub( r'(\S)D', r"\1dd"   , translit )
        translit = re.sub( r'(\S)F', r"\1S3S3" , translit ) # 3 = '
        translit = re.sub( r'F'    , r"S'"     , translit )
        translit = re.sub( r'f'    , r"s"      , translit )
        translit = re.sub( r'(\S)G', r"\1gg"   , translit )
        translit = re.sub( r'(\S)H', r"\1h"    , translit )
        translit = re.sub( r'(\S)J', r"\1t2t2" , translit ) # 2 = .
        translit = re.sub( r'j'    , r"t2"     , translit ) # 2 = .
        translit = re.sub( r'(\S)K', r"\1kk"   , translit )
        translit = re.sub( r'(\S)L', r"\1ll"   , translit )
        translit = re.sub( r'(\S)M', r"\1mm"   , translit )
        translit = re.sub( r'(\S)N', r"\1nn"   , translit )
        # lowercase 'p' must be converted before uppercase 'P" is converted to 'pp'!
        translit = re.sub( r'p'    , r"p4"     , translit ) # 4 = h
        translit = re.sub( r'(\S)P', r"\1pp"   , translit )
        # lowercase 'q' must be converted before uppercase 'Q" is converted to 'qq'!
        translit = re.sub( r'q'    , r"q4"     , translit ) # 4 = h
        translit = re.sub( r'(\S)Q', r"\1qq"   , translit )
        translit = re.sub( r'(\S)R', r"\1rr"   , translit )
        translit = re.sub( r'(\S)S', r"\1ss"   , translit )
        translit = re.sub( r'(\S)T', r"\1tt"   , translit )
        translit = re.sub( r'(\S)V', r"\1s~s~" , translit )
        translit = re.sub( r'V'    , r"S~"     , translit )
        translit = re.sub( r'v'    , r"s~"     , translit )
        translit = re.sub( r'\bW'  , r"6"      , translit ) # 6 = wu
        translit = re.sub( r'x'    , r"h2"     , translit ) # 2 = .
        translit = re.sub( r'(\S)Y', r"\1yy"   , translit )
        translit = re.sub( r'(\S)Z', r"\1zz"   , translit )

        # Vowels after consonants (or may undergo further transliteration!
        # Unchangeably long vowels.
        translit = re.sub( r'W'         , r"7"   , translit ) # 7 = suruq waw
        translit = re.sub( r'([\:\;])y' , r"8"   , translit ) # 8 = qamets yod
        translit = re.sub( r'(e)y'      , r"9"   , translit ) # 9 = sere yod
        translit = re.sub( r'(,)y'      , r"10"  , translit ) # 10 = segol yod
        translit = re.sub( r'(i)y'      , r"11"  , translit ) # 11 = hireq yod

        # English vowel characters before other vowels else may undergo
        # further transliteration!
        translit = re.sub( r'-'      , r"5"  , translit ) # 5 = maqqep <<<< before expand other '-'!
        translit = re.sub( r'[Ee]'   , r"e-" , translit )
        translit = re.sub( r'[Ii]'   , r"i"  , translit )
        translit = re.sub( r'[Oo]'   , r"o-" , translit )
        translit = re.sub( r'[Uu]'   , r"u-" , translit )

        # Hebrew Vowels after consonants (or may undergo further transliteration!
        was = translit
        translit = re.sub( r'[\:\;]' , r"a"  , translit )
        translit = re.sub( r'\\'     , r"o~" , translit )
        translit = re.sub( r'"'      , r"a-" , translit )
        translit = re.sub( r"'"      , r"a-" , translit )
        #translit = re.sub( r'[<,]'   , r"e"  , translit )
        translit = re.sub( r','      , r"e"  , translit )
        translit = re.sub( r'[>.]'   , r"e^^" , translit )
        translit = re.sub( r'[?/]'   , r"e~" , translit )
        translit = re.sub( r'\{'     , r"o-" , translit )
        translit = re.sub( r'[\}\]]' , r"a~" , translit )

        # Restore conversions preserved as numbers.
        # LONGER ONES FIRST!
        translit = re.sub( r'10' , r"e^"  , translit ) # 10 = segol yod
        translit = re.sub( r'11' , r"i^"  , translit ) # 11 = hireq yod
        translit = re.sub( r'12' , r"wo"  , translit ) # 12 = wo-
        translit = re.sub( r'9'  , r"e^"  , translit ) # 9 = sere yod
        translit = re.sub( r'8'  , r"a^"  , translit ) # 8 = qamets yod
        translit = re.sub( r'7'  , r"u^"   , translit ) # 7 = suruq waw
        translit = re.sub( r'6'  , r"wu"   , translit ) # 6 = wu-
        translit = re.sub( r'4'  , r"h"    , translit ) # 4 = h
        translit = re.sub( r'3'  , r"'"    , translit ) # 3 = '
        translit = re.sub( r'2'  , r"."    , translit ) # 2 = .

        # Some auto-conversion before stripping all accents.
        translit = re.sub( 'h\.'  , r"ch"    , translit )
        translit = re.sub( 's\.'  , r"ts"    , translit )
        translit = re.sub( r's~'  , r"sh"    , translit )

        # Strip transliteration accents.
        translit = re.sub( r'5'  , r"-"    , translit ) # 5 = maqqep
        translit = re.sub( r'1'  , r"`"    , translit ) # 1 = `

        if 0 and translit != input :
            print 'TRANSLIT INPUT ', input
            print 'TRANSLIT OUTPUT', output

    return translit

############################################################################
# Represent transliteration using HTML unicode.
############################################################################
# See https://www.alanwood.net/demos/ent4_frame.html
def translitToUnicodeHtml( text ) :
    return translit( text )

def translitToUnicodeRtf( text ) :
    text = translitToUnicodeHtml( text )
    text = htmlToRtfUnicode( text )
    return text

def translit( text ) :

    # Must be first (due to &).
    text = re.sub( '\&', '<sup>e</sup>', text ) # superscript e (segol)
    text = re.sub( 'e\+', '<sup>e</sup>', text ) # superscript e (segol)
    text = re.sub( 'e\^\^', '<sup>e</sup>', text ) # superscript e (segol)

    # Acute.
    text = re.sub( 'A\^', '&#194;', text )
    text = re.sub( 'E\^', '&#202;', text )
    text = re.sub( 'I\^', '&#206;', text )
    text = re.sub( 'O\^', '&#212;', text )
    text = re.sub( "U\^", '&#219;', text )
    text = re.sub( "S'", '&#346;', text )

    text = re.sub( 'a\^', '&#226;', text )
    text = re.sub( 'e\^', '&#234;', text )
    text = re.sub( 'i\^', '&#238;', text )
    text = re.sub( 'o\^', '&#244;', text )
    text = re.sub( "u\^", '&#251;', text )
    text = re.sub( "s'", '&#347;', text )

    # Macron
    text = re.sub( 'A-', '&#256;', text )
    text = re.sub( 'A_', '&#256;', text )
    text = re.sub( 'E-', '&#274;', text )
    text = re.sub( 'E_', '&#274;', text )
    text = re.sub( 'I-', '&#298;', text )
    text = re.sub( 'I_', '&#298;', text )
    text = re.sub( 'O-', '&#332;', text )
    text = re.sub( 'O_', '&#332;', text )
    text = re.sub( 'U-', '&#362;', text )
    text = re.sub( 'U_', '&#362;', text )

    text = re.sub( 'a-', '&#257;', text )
    text = re.sub( 'a_', '&#257;', text )
    text = re.sub( 'e-', '&#275;', text )
    text = re.sub( 'e_', '&#275;', text )
    text = re.sub( 'i-', '&#299;', text )
    text = re.sub( 'i_', '&#299;', text )
    text = re.sub( 'o-', '&#333;', text )
    text = re.sub( 'o_', '&#333;', text )
    text = re.sub( 'u-', '&#363;', text )
    text = re.sub( 'u_', '&#363;', text )

    # Breve
    text = re.sub( 'A~', '&#258;', text )
    text = re.sub( 'E~', '&#282;', text )
    text = re.sub( 'I~', '&#300;', text )
    text = re.sub( 'O~', '&#334;', text )
    text = re.sub( 'U~', '&#364;', text )
    text = re.sub( "S~", '&#352;', text )

    text = re.sub( 'a~', '&#259;', text )
    text = re.sub( 'e~', '&#283;', text )
    text = re.sub( 'i~', '&#300;', text )
    text = re.sub( 'o~', '&#335;', text )
    text = re.sub( "u~", '&#365;', text )
    text = re.sub( "s~", '&#353;', text )

    text = re.sub( 'g\.', '&#289;', text )

    text = re.sub( "A\.", '&#7840;', text )
    text = re.sub( "a\.", '&#7841;', text )
    text = re.sub( "E\.", '&#278;', text )
    text = re.sub( "e\.", '&#279;', text )
    text = re.sub( "G\.", '&#288;', text )
    text = re.sub( "g\.", '&#289;', text )
    text = re.sub( "I\.", '&#7882;', text )
    text = re.sub( "i\.", '&#7883;', text )
    text = re.sub( "O\.", '&#7884;', text )
    text = re.sub( "o\.", '&#7885;', text )
    text = re.sub( "U\.", '&#7908;', text )
    text = re.sub( "u\.", '&#7909;', text )

    text = re.sub( 'g-', "g&#772", text ) #
    text = re.sub( 'p-', "g&#772", text ) #

    text = re.sub( 'h\.', "ch", text ) #
    text = re.sub( 's\.', "ts", text ) #


    # Accent near aleph.
    text = re.sub( '%146', "&#146", text ) # aleph
    text = re.sub( '%145', "&#145", text ) # ayin

    # These are not ideal (use overbars instead of
    # underbars or are not implemented as a single character).
    #text = re.sub( 't-', "t&#772", text ) #
    #text = re.sub( 'd-', "d&#772", text ) #
    #text = re.sub( 'k-', "k&#772", text ) #
    text = re.sub( 't-', "t&#175", text ) #
    text = re.sub( 'd-', "d&#175", text ) #
    text = re.sub( 'k-', "k&#175", text ) #
    text = re.sub( 't\.', "t.", text ) # NOT IMPLEMENTED YET


    return text

# If module is launched as a program, perform indexing using specified files.
if __name__ == '__main__' :
    print 'HEBREW KEYWORD TEST'
    test = '{alep} {bet} {gimel} {dalet} {he} {waw} {zayin} {het} {tet} {yod} {kap} {lamed} {mem} {num} {samek} {ayin} {pe} {sade} {qop} {res} {sin} {shin} {taw} {qames} {hireq} {holem} {segol}'
    print test
    print expand_keywords( test )
    print 

    print 'FINAL CHARACTER TEST'
    test1 = "changed: {g qeos qeos} {h ak ak} {h an an} {h am am} {h ap ap} {h ac ac} unchanged: {g s s} {h k k} {h m m} {h n n} {h p p} {h c c} {h m m} {h q]k'l:t.hW}"
    print test1
    print finals( test1 )

